@@ -80,7 +80,7 @@ SpatialQuantile.ppp <- function(X, prob=0.5, sigma=NULL, ...,
   }
   #' numerical weights 
   if(!is.null(weights)) {
-    check.nvector(weights, nX)
+    check.nvector(weights, nX, vname="weights")
     if(any(weights < 0)) stop("Negative weights are not permitted")
     if(sum(weights) < .Machine$double.eps)
       stop("Weights are numerically zero; quantiles are undefined", call.=FALSE)
@@ -114,7 +114,8 @@ SpatialQuantile.ppp <- function(X, prob=0.5, sigma=NULL, ...,
                   "sigma is probably too small"),
             call.=FALSE)
     #' apply l'Hopital's Rule at the problem locations
-    Z[Bad] <- nnmark(X, at=at, xy=LX)[Bad]
+    tiesfun <- function(x, p=prob, ty=type) {unname(quantile(x, probs=p, type=ty))}
+    Z[Bad] <- nnmark(X, at=at, xy=LX, ties=tiesfun)[Bad]
     Good <- !Bad
   }
   #' compute
